############################ Copyrights and license ############################
#                                                                              #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Hugo <hugovk@users.noreply.github.com>                        #
# Copyright 2018 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Christoph Reiter <reiter.christoph@gmail.com>                 #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2024 iarspider <iarspider@gmail.com>                               #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from datetime import datetime, timezone

import github

from . import Framework


# Replay data is forged to simulate bad things returned by Github
class BadAttributes(Framework.TestCase):
    def testBadSimpleAttribute(self):
        user = self.g.get_user("klmitch")
        self.assertEqual(
            user.created_at,
            datetime(2011, 3, 23, 15, 42, 9, tzinfo=timezone.utc),
        )

        with self.assertRaises(github.BadAttributeException) as raisedexp:
            user.name
        self.assertEqual(raisedexp.exception.actual_value, 42)
        self.assertEqual(raisedexp.exception.expected_type, str)
        self.assertEqual(raisedexp.exception.transformation_exception, None)

    def testBadAttributeTransformation(self):
        user = self.g.get_user("klmitch")
        self.assertEqual(user.name, "Kevin L. Mitchell")

        with self.assertRaises(github.BadAttributeException) as raisedexp:
            user.created_at
        self.assertEqual(raisedexp.exception.actual_value, "foobar")
        self.assertEqual(raisedexp.exception.expected_type, str)
        self.assertEqual(raisedexp.exception.transformation_exception.__class__, ValueError)
        self.assertEqual(
            raisedexp.exception.transformation_exception.args,
            ("Invalid isoformat string: 'foobar'",),
        )

    def testBadTransformedAttribute(self):
        user = self.g.get_user("klmitch")
        self.assertEqual(user.name, "Kevin L. Mitchell")

        with self.assertRaises(github.BadAttributeException) as raisedexp:
            user.updated_at
        self.assertEqual(raisedexp.exception.actual_value, 42)
        self.assertEqual(raisedexp.exception.expected_type, str)
        self.assertEqual(raisedexp.exception.transformation_exception, None)

    def testBadSimpleAttributeInList(self):
        hook = self.g.get_hook("activecollab")
        self.assertEqual(hook.name, "activecollab")

        with self.assertRaises(github.BadAttributeException) as raisedexp:
            hook.events
        self.assertEqual(raisedexp.exception.actual_value, ["push", 42])
        self.assertEqual(raisedexp.exception.expected_type, [str])
        self.assertEqual(raisedexp.exception.transformation_exception, None)

    def testBadAttributeInClassAttribute(self):
        repo = self.g.get_repo("klmitch/turnstile")
        owner = repo.owner
        self.assertEqual(owner.id, 686398)

        with self.assertRaises(github.BadAttributeException) as raisedexp:
            owner.avatar_url
        self.assertEqual(raisedexp.exception.actual_value, 42)

    def testBadTransformedAttributeInList(self):
        commit = self.g.get_repo("klmitch/turnstile", lazy=True).get_commit("38d9082a898d0822b5ccdfd78f3a536e2efa6c26")

        with self.assertRaises(github.BadAttributeException) as raisedexp:
            commit.parents
        self.assertEqual(raisedexp.exception.actual_value, [42])
        self.assertEqual(raisedexp.exception.expected_type, [dict])
        self.assertEqual(raisedexp.exception.transformation_exception, None)

    def testBadTransformedAttributeInDict(self):
        gist = self.g.get_gist("6437766")

        with self.assertRaises(github.BadAttributeException) as raisedexp:
            gist.files
        self.assertEqual(raisedexp.exception.actual_value, {"test.py": 42})
        self.assertEqual(raisedexp.exception.expected_type, {str: dict})
        self.assertEqual(raisedexp.exception.transformation_exception, None)

    def testIssue195(self):
        hooks = self.g.get_hooks()
        # We can loop on all hooks as long as we don't access circleci's events attribute
        self.assertListKeyEqual(
            hooks,
            lambda h: h.name,
            [
                "activecollab",
                "acunote",
                "agilebench",
                "agilezen",
                "amazonsns",
                "apiary",
                "apoio",
                "appharbor",
                "apropos",
                "asana",
                "backlog",
                "bamboo",
                "basecamp",
                "bcx",
                "blimp",
                "boxcar",
                "buddycloud",
                "bugherd",
                "bugly",
                "bugzilla",
                "campfire",
                "cia",
                "circleci",
                "codeclimate",
                "codeportingcsharp2java",
                "codeship",
                "coffeedocinfo",
                "conductor",
                "coop",
                "copperegg",
                "cube",
                "depending",
                "deployhq",
                "devaria",
                "docker",
                "ducksboard",
                "email",
                "firebase",
                "fisheye",
                "flowdock",
                "fogbugz",
                "freckle",
                "friendfeed",
                "gemini",
                "gemnasium",
                "geocommit",
                "getlocalization",
                "gitlive",
                "grmble",
                "grouptalent",
                "grove",
                "habitualist",
                "hakiri",
                "hall",
                "harvest",
                "hipchat",
                "hostedgraphite",
                "hubcap",
                "hubci",
                "humbug",
                "icescrum",
                "irc",
                "irker",
                "ironmq",
                "ironworker",
                "jabber",
                "jaconda",
                "jeapie",
                "jenkins",
                "jenkinsgit",
                "jira",
                "jqueryplugins",
                "kanbanery",
                "kickoff",
                "leanto",
                "lechat",
                "lighthouse",
                "lingohub",
                "loggly",
                "mantisbt",
                "masterbranch",
                "mqttpub",
                "nma",
                "nodejitsu",
                "notifo",
                "ontime",
                "pachube",
                "packagist",
                "phraseapp",
                "pivotaltracker",
                "planbox",
                "planio",
                "prowl",
                "puppetlinter",
                "pushalot",
                "pushover",
                "pythonpackages",
                "railsbp",
                "railsbrakeman",
                "rally",
                "rapidpush",
                "rationaljazzhub",
                "rationalteamconcert",
                "rdocinfo",
                "readthedocs",
                "redmine",
                "rubyforge",
                "scrumdo",
                "shiningpanda",
                "sifter",
                "simperium",
                "slatebox",
                "snowyevening",
                "socialcast",
                "softlayermessaging",
                "sourcemint",
                "splendidbacon",
                "sprintly",
                "sqsqueue",
                "stackmob",
                "statusnet",
                "talker",
                "targetprocess",
                "tddium",
                "teamcity",
                "tender",
                "tenxer",
                "testpilot",
                "toggl",
                "trac",
                "trajectory",
                "travis",
                "trello",
                "twilio",
                "twitter",
                "unfuddle",
                "web",
                "weblate",
                "webtranslateit",
                "yammer",
                "youtrack",
                "zendesk",
                "zohoprojects",
            ],
        )
        for hook in hooks:
            if hook.name != "circleci":
                hook.events

        for hook in hooks:
            if hook.name == "circleci":
                with self.assertRaises(github.BadAttributeException) as raisedexp:
                    hook.events
        self.assertEqual(
            raisedexp.exception.actual_value,
            [
                [
                    "commit_comment",
                    "create",
                    "delete",
                    "download",
                    "follow",
                    "fork",
                    "fork_apply",
                    "gist",
                    "gollum",
                    "issue_comment",
                    "issues",
                    "member",
                    "public",
                    "pull_request",
                    "pull_request_review_comment",
                    "push",
                    "status",
                    "team_add",
                    "watch",
                ]
            ],
        )
        self.assertEqual(raisedexp.exception.expected_type, [str])
        self.assertEqual(raisedexp.exception.transformation_exception, None)



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 AKFish <akfish@gmail.com>                                     #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Justin Kufro <jkufro@andrew.cmu.edu>                          #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2021 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Trim21 <trim21.me@gmail.com>                                  #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2024 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Tim Gates <tim.gates@iress.com>                               #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime
from typing import Any

from github.GithubObject import Attribute, NonCompletableGithubObject, NotSet


class GitCommitVerification(NonCompletableGithubObject):
    """
    This class represents commit verifications.

    The reference can be found here
    https://docs.github.com/en/rest/commits/commits

    The OpenAPI schema can be found at

    - /components/schemas/file-commit/properties/commit/properties/verification
    - /components/schemas/git-commit/properties/verification
    - /components/schemas/verification

    """

    def _initAttributes(self) -> None:
        self._payload: Attribute[str] = NotSet
        self._reason: Attribute[str] = NotSet
        self._signature: Attribute[str] = NotSet
        self._verified: Attribute[bool] = NotSet
        self._verified_at: Attribute[datetime] = NotSet

    @property
    def payload(self) -> str:
        return self._payload.value

    @property
    def reason(self) -> str:
        return self._reason.value

    @property
    def signature(self) -> str:
        return self._signature.value

    @property
    def verified(self) -> bool:
        return self._verified.value

    @property
    def verified_at(self) -> datetime:
        return self._verified_at.value

    def _useAttributes(self, attributes: dict[str, Any]) -> None:
        if "payload" in attributes:  # pragma no branch
            self._payload = self._makeStringAttribute(attributes["payload"])
        if "reason" in attributes:  # pragma no branch
            self._reason = self._makeStringAttribute(attributes["reason"])
        if "signature" in attributes:  # pragma no branch
            self._signature = self._makeStringAttribute(attributes["signature"])
        if "verified" in attributes:  # pragma no branch
            self._verified = self._makeBoolAttribute(attributes["verified"])
        if "verified_at" in attributes:  # pragma no branch
            self._verified_at = self._makeDatetimeAttribute(attributes["verified_at"])



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Simon <spam@esemi.ru>                                         #
# Copyright 2018 Aaron L. Levine <allevin@sandia.gov>                          #
# Copyright 2018 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class IssueEvent(Framework.TestCase):
    def setUp(self):
        super().setUp()
        repo = self.g.get_repo("PyGithub/PyGithub")

        # From Issue #30
        self.event_subscribed = repo.get_issues_event(16347479)
        self.event_assigned = repo.get_issues_event(16347480)
        self.event_referenced = repo.get_issues_event(16348656)
        self.event_closed = repo.get_issues_event(16351220)
        self.event_labeled = repo.get_issues_event(98136337)

        # From Issue 538
        self.event_mentioned = repo.get_issues_event(1009034767)
        self.event_merged = repo.get_issues_event(1015402964)
        self.event_review_requested = repo.get_issues_event(1011101309)

        # From Issue 857
        self.event_reopened = repo.get_issues_event(1782463023)
        self.event_unassigned = repo.get_issues_event(1782463379)
        self.event_unlabeled = repo.get_issues_event(1782463917)
        self.event_renamed = repo.get_issues_event(1782472556)
        self.event_base_ref_changed = repo.get_issues_event(1782915693)
        self.event_head_ref_deleted = repo.get_issues_event(1782917185)
        self.event_head_ref_restored = repo.get_issues_event(1782917299)
        self.event_milestoned = repo.get_issues_event(1783596418)
        self.event_demilestoned = repo.get_issues_event(1783596452)
        self.event_locked = repo.get_issues_event(1783596743)
        self.event_unlocked = repo.get_issues_event(1783596818)
        self.event_review_dismissed = repo.get_issues_event(1783605084)
        self.event_review_request_removed = repo.get_issues_event(1783779835)
        self.event_marked_as_duplicate = repo.get_issues_event(1783779725)
        self.event_unmarked_as_duplicate = repo.get_issues_event(1789228962)
        self.event_added_to_project = repo.get_issues_event(1791766828)
        self.event_moved_columns_in_project = repo.get_issues_event(1791767766)
        self.event_removed_from_project = repo.get_issues_event(1791768212)

        # From Issue 866
        self.event_converted_note_to_issue = repo.get_issues_event(1791769149)

    def testEvent_subscribed_Attributes(self):
        self.assertEqual(self.event_subscribed.actor.login, "jacquev6")
        self.assertEqual(self.event_subscribed.commit_id, None)
        self.assertEqual(
            self.event_subscribed.created_at,
            datetime(2012, 5, 27, 5, 40, 15, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_subscribed.event, "subscribed")
        self.assertEqual(self.event_subscribed.id, 16347479)
        self.assertEqual(self.event_subscribed.issue.number, 30)
        self.assertEqual(
            self.event_subscribed.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/16347479",
        )
        self.assertEqual(self.event_subscribed.node_id, "MDE1OlN1YnNjcmliZWRFdmVudDE2MzQ3NDc5")
        self.assertEqual(self.event_subscribed.commit_url, None)
        self.assertEqual(self.event_subscribed.label, None)
        self.assertEqual(self.event_subscribed.assignee, None)
        self.assertEqual(self.event_subscribed.assigner, None)
        self.assertEqual(self.event_subscribed.author_association, None)
        self.assertEqual(self.event_subscribed.review_requester, None)
        self.assertEqual(self.event_subscribed.requested_reviewer, None)
        self.assertEqual(self.event_subscribed.requested_team, None)
        self.assertEqual(self.event_subscribed.milestone, None)
        self.assertEqual(self.event_subscribed.performed_via_github_app, None)
        self.assertEqual(self.event_subscribed.project_card, None)
        self.assertEqual(self.event_subscribed.rename, None)
        self.assertEqual(self.event_subscribed.dismissed_review, None)
        self.assertEqual(self.event_subscribed.lock_reason, None)
        self.assertEqual(repr(self.event_subscribed), "IssueEvent(id=16347479)")

    def testEvent_assigned_Attributes(self):
        self.assertEqual(self.event_assigned.actor.login, "jacquev6")
        self.assertEqual(self.event_assigned.commit_id, None)
        self.assertEqual(
            self.event_assigned.created_at,
            datetime(2012, 5, 27, 5, 40, 15, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_assigned.event, "assigned")
        self.assertEqual(self.event_assigned.id, 16347480)
        self.assertEqual(self.event_assigned.issue.number, 30)
        self.assertEqual(
            self.event_assigned.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/16347480",
        )
        self.assertEqual(self.event_assigned.node_id, "MDEzOkFzc2lnbmVkRXZlbnQxNjM0NzQ4MA==")
        self.assertEqual(self.event_assigned.commit_url, None)
        self.assertEqual(self.event_assigned.label, None)
        self.assertEqual(self.event_assigned.assignee.login, "jacquev6")
        self.assertEqual(self.event_assigned.assigner.login, "ghost")
        self.assertEqual(self.event_assigned.author_association, None)
        self.assertEqual(self.event_assigned.review_requester, None)
        self.assertEqual(self.event_assigned.requested_reviewer, None)
        self.assertEqual(self.event_assigned.requested_team, None)
        self.assertEqual(self.event_assigned.milestone, None)
        self.assertEqual(self.event_assigned.performed_via_github_app, None)
        self.assertEqual(self.event_assigned.project_card, None)
        self.assertEqual(self.event_assigned.rename, None)
        self.assertEqual(self.event_assigned.dismissed_review, None)
        self.assertEqual(self.event_assigned.lock_reason, None)
        self.assertEqual(repr(self.event_assigned), "IssueEvent(id=16347480)")

    def testEvent_referenced_Attributes(self):
        self.assertEqual(self.event_referenced.actor.login, "jacquev6")
        self.assertEqual(self.event_referenced.commit_id, "ed866fc43833802ab553e5ff8581c81bb00dd433")
        self.assertEqual(
            self.event_referenced.created_at,
            datetime(2012, 5, 27, 7, 29, 25, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_referenced.event, "referenced")
        self.assertEqual(self.event_referenced.id, 16348656)
        self.assertEqual(self.event_referenced.issue.number, 30)
        self.assertEqual(
            self.event_referenced.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/16348656",
        )
        self.assertEqual(self.event_referenced.node_id, "MDE1OlJlZmVyZW5jZWRFdmVudDE2MzQ4NjU2")
        self.assertEqual(
            self.event_referenced.commit_url,
            "https://api.github.com/repos/PyGithub/PyGithub/commits/ed866fc43833802ab553e5ff8581c81bb00dd433",
        )
        self.assertEqual(self.event_referenced.label, None)
        self.assertEqual(self.event_referenced.assignee, None)
        self.assertEqual(self.event_referenced.assigner, None)
        self.assertEqual(self.event_referenced.author_association, None)
        self.assertEqual(self.event_referenced.review_requester, None)
        self.assertEqual(self.event_referenced.requested_reviewer, None)
        self.assertEqual(self.event_referenced.requested_team, None)
        self.assertEqual(self.event_referenced.milestone, None)
        self.assertEqual(self.event_referenced.performed_via_github_app, None)
        self.assertEqual(self.event_referenced.project_card, None)
        self.assertEqual(self.event_referenced.rename, None)
        self.assertEqual(self.event_referenced.dismissed_review, None)
        self.assertEqual(self.event_referenced.lock_reason, None)
        self.assertEqual(repr(self.event_referenced), "IssueEvent(id=16348656)")

    def testEvent_closed_Attributes(self):
        self.assertEqual(self.event_closed.actor.login, "jacquev6")
        self.assertEqual(self.event_closed.commit_id, None)
        self.assertEqual(
            self.event_closed.created_at,
            datetime(2012, 5, 27, 11, 4, 25, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_closed.event, "closed")
        self.assertEqual(self.event_closed.id, 16351220)
        self.assertEqual(self.event_closed.issue.number, 30)
        self.assertEqual(
            self.event_closed.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/16351220",
        )
        self.assertEqual(self.event_closed.node_id, "MDExOkNsb3NlZEV2ZW50MTYzNTEyMjA=")
        self.assertEqual(self.event_closed.commit_url, None)
        self.assertEqual(self.event_closed.label, None)
        self.assertEqual(self.event_closed.assignee, None)
        self.assertEqual(self.event_closed.assigner, None)
        self.assertEqual(self.event_closed.author_association, None)
        self.assertEqual(self.event_closed.review_requester, None)
        self.assertEqual(self.event_closed.requested_reviewer, None)
        self.assertEqual(self.event_closed.requested_team, None)
        self.assertEqual(self.event_closed.milestone, None)
        self.assertEqual(self.event_closed.performed_via_github_app, None)
        self.assertEqual(self.event_closed.project_card, None)
        self.assertEqual(self.event_closed.rename, None)
        self.assertEqual(self.event_closed.dismissed_review, None)
        self.assertEqual(self.event_closed.lock_reason, None)
        self.assertEqual(repr(self.event_closed), "IssueEvent(id=16351220)")

    def testEvent_labeled_Attributes(self):
        self.assertEqual(self.event_labeled.actor.login, "jacquev6")
        self.assertEqual(self.event_labeled.commit_id, None)
        self.assertEqual(
            self.event_labeled.created_at,
            datetime(2014, 3, 2, 18, 55, 10, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_labeled.event, "labeled")
        self.assertEqual(self.event_labeled.id, 98136337)
        self.assertEqual(self.event_labeled.issue.number, 30)
        self.assertEqual(
            self.event_labeled.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/98136337",
        )
        self.assertEqual(self.event_labeled.node_id, "MDEyOkxhYmVsZWRFdmVudDk4MTM2MzM3")
        self.assertEqual(self.event_labeled.commit_url, None)
        self.assertEqual(self.event_labeled.label.name, "v1")
        self.assertEqual(self.event_labeled.assignee, None)
        self.assertEqual(self.event_labeled.assigner, None)
        self.assertEqual(self.event_labeled.author_association, None)
        self.assertEqual(self.event_labeled.review_requester, None)
        self.assertEqual(self.event_labeled.requested_reviewer, None)
        self.assertEqual(self.event_labeled.requested_team, None)
        self.assertEqual(self.event_labeled.milestone, None)
        self.assertEqual(self.event_labeled.performed_via_github_app, None)
        self.assertEqual(self.event_labeled.project_card, None)
        self.assertEqual(self.event_labeled.rename, None)
        self.assertEqual(self.event_labeled.dismissed_review, None)
        self.assertEqual(self.event_labeled.lock_reason, None)
        self.assertEqual(repr(self.event_labeled), "IssueEvent(id=98136337)")

    def testEvent_mentioned_Attributes(self):
        self.assertEqual(self.event_mentioned.actor.login, "jzelinskie")
        self.assertEqual(self.event_mentioned.commit_id, None)
        self.assertEqual(
            self.event_mentioned.created_at,
            datetime(2017, 3, 21, 17, 30, 14, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_mentioned.event, "mentioned")
        self.assertEqual(self.event_mentioned.id, 1009034767)
        self.assertEqual(self.event_mentioned.issue.number, 538)
        self.assertEqual(
            self.event_mentioned.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1009034767",
        )
        self.assertEqual(self.event_mentioned.node_id, "MDE0Ok1lbnRpb25lZEV2ZW50MTAwOTAzNDc2Nw==")
        self.assertEqual(self.event_mentioned.commit_url, None)
        self.assertEqual(self.event_mentioned.label, None)
        self.assertEqual(self.event_mentioned.assignee, None)
        self.assertEqual(self.event_mentioned.assigner, None)
        self.assertEqual(self.event_mentioned.author_association, None)
        self.assertEqual(self.event_mentioned.review_requester, None)
        self.assertEqual(self.event_mentioned.requested_reviewer, None)
        self.assertEqual(self.event_mentioned.requested_team, None)
        self.assertEqual(self.event_mentioned.milestone, None)
        self.assertEqual(self.event_mentioned.performed_via_github_app, None)
        self.assertEqual(self.event_mentioned.project_card, None)
        self.assertEqual(self.event_mentioned.rename, None)
        self.assertEqual(self.event_mentioned.dismissed_review, None)
        self.assertEqual(self.event_mentioned.lock_reason, None)
        self.assertEqual(repr(self.event_mentioned), "IssueEvent(id=1009034767)")

    def testEvent_merged_Attributes(self):
        self.assertEqual(self.event_merged.actor.login, "jzelinskie")
        self.assertEqual(self.event_merged.commit_id, "2525515b094d7425f7018eb5b66171e21c5fbc10")
        self.assertEqual(
            self.event_merged.created_at,
            datetime(2017, 3, 25, 16, 52, 49, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_merged.event, "merged")
        self.assertEqual(self.event_merged.id, 1015402964)
        self.assertEqual(self.event_merged.issue.number, 538)
        self.assertEqual(
            self.event_merged.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1015402964",
        )
        self.assertEqual(self.event_merged.node_id, "MDExOk1lcmdlZEV2ZW50MTAxNTQwMjk2NA==")
        self.assertEqual(
            self.event_merged.commit_url,
            "https://api.github.com/repos/PyGithub/PyGithub/commits/2525515b094d7425f7018eb5b66171e21c5fbc10",
        )
        self.assertEqual(self.event_merged.label, None)
        self.assertEqual(self.event_merged.assignee, None)
        self.assertEqual(self.event_merged.assigner, None)
        self.assertEqual(self.event_merged.author_association, None)
        self.assertEqual(self.event_merged.review_requester, None)
        self.assertEqual(self.event_merged.requested_reviewer, None)
        self.assertEqual(self.event_merged.requested_team, None)
        self.assertEqual(self.event_merged.milestone, None)
        self.assertEqual(self.event_merged.performed_via_github_app, None)
        self.assertEqual(self.event_merged.project_card, None)
        self.assertEqual(self.event_merged.rename, None)
        self.assertEqual(self.event_merged.dismissed_review, None)
        self.assertEqual(self.event_merged.lock_reason, None)
        self.assertEqual(repr(self.event_merged), "IssueEvent(id=1015402964)")

    def testEvent_review_requested_Attributes(self):
        self.assertEqual(self.event_review_requested.actor.login, "jzelinskie")
        self.assertEqual(self.event_review_requested.commit_id, None)
        self.assertEqual(
            self.event_review_requested.created_at,
            datetime(2017, 3, 22, 19, 6, 44, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_review_requested.event, "review_requested")
        self.assertEqual(self.event_review_requested.id, 1011101309)
        self.assertEqual(self.event_review_requested.issue.number, 538)
        self.assertEqual(
            self.event_review_requested.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1011101309",
        )
        self.assertEqual(
            self.event_review_requested.node_id,
            "MDIwOlJldmlld1JlcXVlc3RlZEV2ZW50MTAxMTEwMTMwOQ==",
        )
        self.assertEqual(self.event_review_requested.commit_url, None)
        self.assertEqual(self.event_review_requested.label, None)
        self.assertEqual(self.event_review_requested.assignee, None)
        self.assertEqual(self.event_review_requested.assigner, None)
        self.assertEqual(self.event_review_requested.author_association, None)
        self.assertEqual(self.event_review_requested.review_requester.login, "jzelinskie")
        self.assertEqual(self.event_review_requested.requested_reviewer.login, "jzelinskie")
        self.assertEqual(self.event_review_requested.requested_team, None)
        self.assertEqual(self.event_review_requested.milestone, None)
        self.assertEqual(self.event_review_requested.performed_via_github_app, None)
        self.assertEqual(self.event_review_requested.project_card, None)
        self.assertEqual(self.event_review_requested.rename, None)
        self.assertEqual(self.event_review_requested.dismissed_review, None)
        self.assertEqual(self.event_review_requested.lock_reason, None)
        self.assertEqual(repr(self.event_review_requested), "IssueEvent(id=1011101309)")

    def testEvent_reopened_Attributes(self):
        self.assertEqual(self.event_reopened.actor.login, "sfdye")
        self.assertEqual(self.event_reopened.commit_id, None)
        self.assertEqual(
            self.event_reopened.created_at,
            datetime(2018, 8, 10, 13, 10, 9, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_reopened.event, "reopened")
        self.assertEqual(self.event_reopened.id, 1782463023)
        self.assertEqual(self.event_reopened.issue.number, 857)
        self.assertEqual(
            self.event_reopened.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1782463023",
        )
        self.assertEqual(self.event_reopened.node_id, "MDEzOlJlb3BlbmVkRXZlbnQxNzgyNDYzMDIz")
        self.assertEqual(self.event_reopened.commit_url, None)
        self.assertEqual(self.event_reopened.label, None)
        self.assertEqual(self.event_reopened.assignee, None)
        self.assertEqual(self.event_reopened.assigner, None)
        self.assertEqual(self.event_reopened.author_association, None)
        self.assertEqual(self.event_reopened.review_requester, None)
        self.assertEqual(self.event_reopened.requested_reviewer, None)
        self.assertEqual(self.event_reopened.requested_team, None)
        self.assertEqual(self.event_reopened.milestone, None)
        self.assertEqual(self.event_reopened.performed_via_github_app, None)
        self.assertEqual(self.event_reopened.project_card, None)
        self.assertEqual(self.event_reopened.rename, None)
        self.assertEqual(self.event_reopened.dismissed_review, None)
        self.assertEqual(self.event_reopened.lock_reason, None)
        self.assertEqual(repr(self.event_reopened), "IssueEvent(id=1782463023)")

    def testEvent_unassigned_Attributes(self):
        self.assertEqual(self.event_unassigned.actor.login, "sfdye")
        self.assertEqual(self.event_unassigned.commit_id, None)
        self.assertEqual(
            self.event_unassigned.created_at,
            datetime(2018, 8, 10, 13, 10, 21, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_unassigned.event, "unassigned")
        self.assertEqual(self.event_unassigned.id, 1782463379)
        self.assertEqual(self.event_unassigned.issue.number, 857)
        self.assertEqual(
            self.event_unassigned.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1782463379",
        )
        self.assertEqual(self.event_unassigned.node_id, "MDE1OlVuYXNzaWduZWRFdmVudDE3ODI0NjMzNzk=")
        self.assertEqual(self.event_unassigned.commit_url, None)
        self.assertEqual(self.event_unassigned.label, None)
        self.assertEqual(self.event_unassigned.assignee.login, "sfdye")
        self.assertEqual(self.event_unassigned.assigner.login, "sfdye")
        self.assertEqual(self.event_unassigned.author_association, None)
        self.assertEqual(self.event_unassigned.review_requester, None)
        self.assertEqual(self.event_unassigned.requested_reviewer, None)
        self.assertEqual(self.event_unassigned.requested_team, None)
        self.assertEqual(self.event_unassigned.milestone, None)
        self.assertEqual(self.event_unassigned.performed_via_github_app, None)
        self.assertEqual(self.event_unassigned.project_card, None)
        self.assertEqual(self.event_unassigned.rename, None)
        self.assertEqual(self.event_unassigned.dismissed_review, None)
        self.assertEqual(self.event_unassigned.lock_reason, None)
        self.assertEqual(repr(self.event_unassigned), "IssueEvent(id=1782463379)")

    def testEvent_unlabeled_Attributes(self):
        self.assertEqual(self.event_unlabeled.actor.login, "sfdye")
        self.assertEqual(self.event_unlabeled.commit_id, None)
        self.assertEqual(
            self.event_unlabeled.created_at,
            datetime(2018, 8, 10, 13, 10, 38, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_unlabeled.event, "unlabeled")
        self.assertEqual(self.event_unlabeled.id, 1782463917)
        self.assertEqual(self.event_unlabeled.issue.number, 857)
        self.assertEqual(
            self.event_unlabeled.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1782463917",
        )
        self.assertEqual(self.event_unlabeled.node_id, "MDE0OlVubGFiZWxlZEV2ZW50MTc4MjQ2MzkxNw==")
        self.assertEqual(self.event_unlabeled.commit_url, None)
        self.assertEqual(self.event_unlabeled.label.name, "improvement")
        self.assertEqual(self.event_unlabeled.assignee, None)
        self.assertEqual(self.event_unlabeled.assigner, None)
        self.assertEqual(self.event_unlabeled.author_association, None)
        self.assertEqual(self.event_unlabeled.review_requester, None)
        self.assertEqual(self.event_unlabeled.requested_reviewer, None)
        self.assertEqual(self.event_unlabeled.requested_team, None)
        self.assertEqual(self.event_unlabeled.milestone, None)
        self.assertEqual(self.event_unlabeled.performed_via_github_app, None)
        self.assertEqual(self.event_unlabeled.project_card, None)
        self.assertEqual(self.event_unlabeled.rename, None)
        self.assertEqual(self.event_unlabeled.dismissed_review, None)
        self.assertEqual(self.event_unlabeled.lock_reason, None)
        self.assertEqual(repr(self.event_unlabeled), "IssueEvent(id=1782463917)")

    def testEvent_renamed_Attributes(self):
        self.assertEqual(self.event_renamed.actor.login, "sfdye")
        self.assertEqual(self.event_renamed.commit_id, None)
        self.assertEqual(
            self.event_renamed.created_at,
            datetime(2018, 8, 10, 13, 15, 18, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_renamed.event, "renamed")
        self.assertEqual(self.event_renamed.id, 1782472556)
        self.assertEqual(self.event_renamed.issue.number, 857)
        self.assertEqual(
            self.event_renamed.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1782472556",
        )
        self.assertEqual(self.event_renamed.node_id, "MDE3OlJlbmFtZWRUaXRsZUV2ZW50MTc4MjQ3MjU1Ng==")
        self.assertEqual(self.event_renamed.commit_url, None)
        self.assertEqual(self.event_renamed.label, None)
        self.assertEqual(self.event_renamed.assignee, None)
        self.assertEqual(self.event_renamed.assigner, None)
        self.assertEqual(self.event_renamed.author_association, None)
        self.assertEqual(self.event_renamed.review_requester, None)
        self.assertEqual(self.event_renamed.requested_reviewer, None)
        self.assertEqual(self.event_renamed.requested_team, None)
        self.assertEqual(self.event_renamed.milestone, None)
        self.assertEqual(self.event_renamed.performed_via_github_app, None)
        self.assertEqual(self.event_renamed.project_card, None)
        self.assertEqual(
            self.event_renamed.rename,
            {
                "to": "Adding new attributes to IssueEvent",
                "from": "Adding new attributes to IssueEvent Object (DO NOT MERGE - SEE NOTES)",
            },
        )
        self.assertEqual(self.event_renamed.dismissed_review, None)
        self.assertEqual(self.event_renamed.lock_reason, None)
        self.assertEqual(repr(self.event_renamed), "IssueEvent(id=1782472556)")

    def testEvent_base_ref_changed_Attributes(self):
        self.assertEqual(self.event_base_ref_changed.actor.login, "allevin")
        self.assertEqual(self.event_base_ref_changed.commit_id, None)
        self.assertEqual(
            self.event_base_ref_changed.created_at,
            datetime(2018, 8, 10, 16, 38, 22, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_base_ref_changed.event, "base_ref_changed")
        self.assertEqual(self.event_base_ref_changed.id, 1782915693)
        self.assertEqual(self.event_base_ref_changed.issue.number, 857)
        self.assertEqual(
            self.event_base_ref_changed.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1782915693",
        )
        self.assertEqual(
            self.event_base_ref_changed.node_id,
            "MDE5OkJhc2VSZWZDaGFuZ2VkRXZlbnQxNzgyOTE1Njkz",
        )
        self.assertEqual(self.event_base_ref_changed.commit_url, None)
        self.assertEqual(self.event_base_ref_changed.label, None)
        self.assertEqual(self.event_base_ref_changed.assignee, None)
        self.assertEqual(self.event_base_ref_changed.assigner, None)
        self.assertEqual(self.event_base_ref_changed.author_association, None)
        self.assertEqual(self.event_base_ref_changed.review_requester, None)
        self.assertEqual(self.event_base_ref_changed.requested_reviewer, None)
        self.assertEqual(self.event_base_ref_changed.requested_team, None)
        self.assertEqual(self.event_base_ref_changed.milestone, None)
        self.assertEqual(self.event_base_ref_changed.performed_via_github_app, None)
        self.assertEqual(self.event_base_ref_changed.project_card, None)
        self.assertEqual(self.event_base_ref_changed.rename, None)
        self.assertEqual(self.event_base_ref_changed.dismissed_review, None)
        self.assertEqual(self.event_base_ref_changed.lock_reason, None)
        self.assertEqual(repr(self.event_base_ref_changed), "IssueEvent(id=1782915693)")

    def testEvent_head_ref_deleted_Attributes(self):
        self.assertEqual(self.event_head_ref_deleted.actor.login, "allevin")
        self.assertEqual(self.event_head_ref_deleted.commit_id, None)
        self.assertEqual(
            self.event_head_ref_deleted.created_at,
            datetime(2018, 8, 10, 16, 39, 20, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_head_ref_deleted.event, "head_ref_deleted")
        self.assertEqual(self.event_head_ref_deleted.id, 1782917185)
        self.assertEqual(self.event_head_ref_deleted.issue.number, 857)
        self.assertEqual(
            self.event_head_ref_deleted.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1782917185",
        )
        self.assertEqual(
            self.event_head_ref_deleted.node_id,
            "MDE5OkhlYWRSZWZEZWxldGVkRXZlbnQxNzgyOTE3MTg1",
        )
        self.assertEqual(self.event_head_ref_deleted.commit_url, None)
        self.assertEqual(self.event_head_ref_deleted.label, None)
        self.assertEqual(self.event_head_ref_deleted.assignee, None)
        self.assertEqual(self.event_head_ref_deleted.assigner, None)
        self.assertEqual(self.event_head_ref_deleted.author_association, None)
        self.assertEqual(self.event_head_ref_deleted.review_requester, None)
        self.assertEqual(self.event_head_ref_deleted.requested_reviewer, None)
        self.assertEqual(self.event_head_ref_deleted.requested_team, None)
        self.assertEqual(self.event_head_ref_deleted.milestone, None)
        self.assertEqual(self.event_head_ref_deleted.performed_via_github_app, None)
        self.assertEqual(self.event_head_ref_deleted.project_card, None)
        self.assertEqual(self.event_head_ref_deleted.rename, None)
        self.assertEqual(self.event_head_ref_deleted.dismissed_review, None)
        self.assertEqual(self.event_head_ref_deleted.lock_reason, None)
        self.assertEqual(repr(self.event_head_ref_deleted), "IssueEvent(id=1782917185)")

    def testEvent_head_ref_restored_Attributes(self):
        self.assertEqual(self.event_head_ref_restored.actor.login, "allevin")
        self.assertEqual(self.event_head_ref_restored.commit_id, None)
        self.assertEqual(
            self.event_head_ref_restored.created_at,
            datetime(2018, 8, 10, 16, 39, 23, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_head_ref_restored.event, "head_ref_restored")
        self.assertEqual(self.event_head_ref_restored.id, 1782917299)
        self.assertEqual(self.event_head_ref_restored.issue.number, 857)
        self.assertEqual(
            self.event_head_ref_restored.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1782917299",
        )
        self.assertEqual(
            self.event_head_ref_restored.node_id,
            "MDIwOkhlYWRSZWZSZXN0b3JlZEV2ZW50MTc4MjkxNzI5OQ==",
        )
        self.assertEqual(self.event_head_ref_restored.commit_url, None)
        self.assertEqual(self.event_head_ref_restored.label, None)
        self.assertEqual(self.event_head_ref_restored.assignee, None)
        self.assertEqual(self.event_head_ref_restored.assigner, None)
        self.assertEqual(self.event_head_ref_restored.author_association, None)
        self.assertEqual(self.event_head_ref_restored.review_requester, None)
        self.assertEqual(self.event_head_ref_restored.requested_reviewer, None)
        self.assertEqual(self.event_head_ref_restored.requested_team, None)
        self.assertEqual(self.event_head_ref_restored.milestone, None)
        self.assertEqual(self.event_head_ref_restored.performed_via_github_app, None)
        self.assertEqual(self.event_head_ref_restored.project_card, None)
        self.assertEqual(self.event_head_ref_restored.rename, None)
        self.assertEqual(self.event_head_ref_restored.dismissed_review, None)
        self.assertEqual(self.event_head_ref_restored.lock_reason, None)
        self.assertEqual(repr(self.event_head_ref_restored), "IssueEvent(id=1782917299)")

    def testEvent_milestoned_Attributes(self):
        self.assertEqual(self.event_milestoned.actor.login, "sfdye")
        self.assertEqual(self.event_milestoned.commit_id, None)
        self.assertEqual(
            self.event_milestoned.created_at,
            datetime(2018, 8, 11, 0, 46, 19, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_milestoned.event, "milestoned")
        self.assertEqual(self.event_milestoned.id, 1783596418)
        self.assertEqual(self.event_milestoned.issue.number, 857)
        self.assertEqual(
            self.event_milestoned.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1783596418",
        )
        self.assertEqual(self.event_milestoned.node_id, "MDE1Ok1pbGVzdG9uZWRFdmVudDE3ODM1OTY0MTg=")
        self.assertEqual(self.event_milestoned.commit_url, None)
        self.assertEqual(self.event_milestoned.label, None)
        self.assertEqual(self.event_milestoned.assignee, None)
        self.assertEqual(self.event_milestoned.assigner, None)
        self.assertEqual(self.event_milestoned.author_association, None)
        self.assertEqual(self.event_milestoned.review_requester, None)
        self.assertEqual(self.event_milestoned.requested_reviewer, None)
        self.assertEqual(self.event_milestoned.requested_team, None)
        self.assertEqual(self.event_milestoned.milestone.title, "Version 1.25.0")
        self.assertEqual(self.event_milestoned.performed_via_github_app, None)
        self.assertEqual(self.event_milestoned.project_card, None)
        self.assertEqual(self.event_milestoned.rename, None)
        self.assertEqual(self.event_milestoned.dismissed_review, None)
        self.assertEqual(self.event_milestoned.lock_reason, None)
        self.assertEqual(repr(self.event_milestoned), "IssueEvent(id=1783596418)")

    def testEvent_demilestoned_Attributes(self):
        self.assertEqual(self.event_demilestoned.actor.login, "sfdye")
        self.assertEqual(self.event_demilestoned.commit_id, None)
        self.assertEqual(
            self.event_demilestoned.created_at,
            datetime(2018, 8, 11, 0, 46, 22, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_demilestoned.event, "demilestoned")
        self.assertEqual(self.event_demilestoned.id, 1783596452)
        self.assertEqual(self.event_demilestoned.issue.number, 857)
        self.assertEqual(
            self.event_demilestoned.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1783596452",
        )
        self.assertEqual(
            self.event_demilestoned.node_id,
            "MDE3OkRlbWlsZXN0b25lZEV2ZW50MTc4MzU5NjQ1Mg==",
        )
        self.assertEqual(self.event_demilestoned.commit_url, None)
        self.assertEqual(self.event_demilestoned.label, None)
        self.assertEqual(self.event_demilestoned.assignee, None)
        self.assertEqual(self.event_demilestoned.assigner, None)
        self.assertEqual(self.event_demilestoned.author_association, None)
        self.assertEqual(self.event_demilestoned.review_requester, None)
        self.assertEqual(self.event_demilestoned.requested_reviewer, None)
        self.assertEqual(self.event_demilestoned.requested_team, None)
        self.assertEqual(self.event_demilestoned.milestone.title, "Version 1.25.0")
        self.assertEqual(self.event_demilestoned.performed_via_github_app, None)
        self.assertEqual(self.event_demilestoned.project_card, None)
        self.assertEqual(self.event_demilestoned.rename, None)
        self.assertEqual(self.event_demilestoned.dismissed_review, None)
        self.assertEqual(self.event_demilestoned.lock_reason, None)
        self.assertEqual(repr(self.event_demilestoned), "IssueEvent(id=1783596452)")

    def testEvent_locked_Attributes(self):
        self.assertEqual(self.event_locked.actor.login, "sfdye")
        self.assertEqual(self.event_locked.commit_id, None)
        self.assertEqual(
            self.event_locked.created_at,
            datetime(2018, 8, 11, 0, 46, 56, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_locked.event, "locked")
        self.assertEqual(self.event_locked.id, 1783596743)
        self.assertEqual(self.event_locked.issue.number, 857)
        self.assertEqual(
            self.event_locked.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1783596743",
        )
        self.assertEqual(self.event_locked.node_id, "MDExOkxvY2tlZEV2ZW50MTc4MzU5Njc0Mw==")
        self.assertEqual(self.event_locked.commit_url, None)
        self.assertEqual(self.event_locked.label, None)
        self.assertEqual(self.event_locked.assignee, None)
        self.assertEqual(self.event_locked.assigner, None)
        self.assertEqual(self.event_locked.author_association, None)
        self.assertEqual(self.event_locked.review_requester, None)
        self.assertEqual(self.event_locked.requested_reviewer, None)
        self.assertEqual(self.event_locked.requested_team, None)
        self.assertEqual(self.event_locked.milestone, None)
        self.assertEqual(self.event_locked.performed_via_github_app, None)
        self.assertEqual(self.event_locked.project_card, None)
        self.assertEqual(self.event_locked.rename, None)
        self.assertEqual(self.event_locked.dismissed_review, None)
        self.assertEqual(self.event_locked.lock_reason, "too heated")
        self.assertEqual(repr(self.event_locked), "IssueEvent(id=1783596743)")

    def testEvent_unlocked_Attributes(self):
        self.assertEqual(self.event_unlocked.actor.login, "sfdye")
        self.assertEqual(self.event_unlocked.commit_id, None)
        self.assertEqual(
            self.event_unlocked.created_at,
            datetime(2018, 8, 11, 0, 47, 7, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_unlocked.event, "unlocked")
        self.assertEqual(self.event_unlocked.id, 1783596818)
        self.assertEqual(self.event_unlocked.issue.number, 857)
        self.assertEqual(
            self.event_unlocked.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1783596818",
        )
        self.assertEqual(self.event_unlocked.node_id, "MDEzOlVubG9ja2VkRXZlbnQxNzgzNTk2ODE4")
        self.assertEqual(self.event_unlocked.commit_url, None)
        self.assertEqual(self.event_unlocked.label, None)
        self.assertEqual(self.event_unlocked.assignee, None)
        self.assertEqual(self.event_unlocked.assigner, None)
        self.assertEqual(self.event_unlocked.author_association, None)
        self.assertEqual(self.event_unlocked.review_requester, None)
        self.assertEqual(self.event_unlocked.requested_reviewer, None)
        self.assertEqual(self.event_unlocked.requested_team, None)
        self.assertEqual(self.event_unlocked.milestone, None)
        self.assertEqual(self.event_unlocked.performed_via_github_app, None)
        self.assertEqual(self.event_unlocked.project_card, None)
        self.assertEqual(self.event_unlocked.rename, None)
        self.assertEqual(self.event_unlocked.dismissed_review, None)
        self.assertEqual(self.event_unlocked.lock_reason, None)
        self.assertEqual(repr(self.event_unlocked), "IssueEvent(id=1783596818)")

    def testEvent_review_dismissed_Attributes(self):
        self.assertEqual(self.event_review_dismissed.actor.login, "sfdye")
        self.assertEqual(self.event_review_dismissed.commit_id, None)
        self.assertEqual(
            self.event_review_dismissed.created_at,
            datetime(2018, 8, 11, 1, 7, 10, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_review_dismissed.event, "review_dismissed")
        self.assertEqual(self.event_review_dismissed.id, 1783605084)
        self.assertEqual(self.event_review_dismissed.issue.number, 857)
        self.assertEqual(
            self.event_review_dismissed.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1783605084",
        )
        self.assertEqual(
            self.event_review_dismissed.node_id,
            "MDIwOlJldmlld0Rpc21pc3NlZEV2ZW50MTc4MzYwNTA4NA==",
        )
        self.assertEqual(self.event_review_dismissed.commit_url, None)
        self.assertEqual(self.event_review_dismissed.label, None)
        self.assertEqual(self.event_review_dismissed.assignee, None)
        self.assertEqual(self.event_review_dismissed.assigner, None)
        self.assertEqual(self.event_review_dismissed.author_association, None)
        self.assertEqual(self.event_review_dismissed.review_requester, None)
        self.assertEqual(self.event_review_dismissed.requested_reviewer, None)
        self.assertEqual(self.event_review_dismissed.requested_team, None)
        self.assertEqual(self.event_review_dismissed.milestone, None)
        self.assertEqual(self.event_review_dismissed.performed_via_github_app, None)
        self.assertEqual(self.event_review_dismissed.project_card, None)
        self.assertEqual(self.event_review_dismissed.rename, None)
        self.assertEqual(
            self.event_review_dismissed.dismissed_review,
            {
                "dismissal_message": "dismiss",
                "state": "changes_requested",
                "review_id": 145431295,
            },
        )
        self.assertEqual(self.event_review_dismissed.lock_reason, None)
        self.assertEqual(repr(self.event_review_dismissed), "IssueEvent(id=1783605084)")

    def testEvent_review_request_removed_Attributes(self):
        self.assertEqual(self.event_review_request_removed.actor.login, "sfdye")
        self.assertEqual(self.event_review_request_removed.commit_id, None)
        self.assertEqual(
            self.event_review_request_removed.created_at,
            datetime(2018, 8, 11, 12, 32, 59, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_review_request_removed.event, "review_request_removed")
        self.assertEqual(self.event_review_request_removed.id, 1783779835)
        self.assertEqual(self.event_review_request_removed.issue.number, 857)
        self.assertEqual(
            self.event_review_request_removed.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1783779835",
        )
        self.assertEqual(
            self.event_review_request_removed.node_id,
            "MDI1OlJldmlld1JlcXVlc3RSZW1vdmVkRXZlbnQxNzgzNzc5ODM1",
        )
        self.assertEqual(self.event_review_request_removed.commit_url, None)
        self.assertEqual(self.event_review_request_removed.label, None)
        self.assertEqual(self.event_review_request_removed.assignee, None)
        self.assertEqual(self.event_review_request_removed.assigner, None)
        self.assertEqual(self.event_review_request_removed.author_association, None)
        self.assertEqual(self.event_review_request_removed.review_requester.login, "sfdye")
        self.assertEqual(self.event_review_request_removed.requested_reviewer.login, "jasonwhite")
        self.assertEqual(self.event_review_request_removed.requested_team, None)
        self.assertEqual(self.event_review_request_removed.milestone, None)
        self.assertEqual(self.event_review_request_removed.performed_via_github_app, None)
        self.assertEqual(self.event_review_request_removed.project_card, None)
        self.assertEqual(self.event_review_request_removed.rename, None)
        self.assertEqual(self.event_review_request_removed.dismissed_review, None)
        self.assertEqual(self.event_review_request_removed.lock_reason, None)
        self.assertEqual(repr(self.event_review_request_removed), "IssueEvent(id=1783779835)")

    def testEvent_marked_as_duplicate_Attributes(self):
        self.assertEqual(self.event_marked_as_duplicate.actor.login, "sfdye")
        self.assertEqual(self.event_marked_as_duplicate.commit_id, None)
        self.assertEqual(
            self.event_marked_as_duplicate.created_at,
            datetime(2018, 8, 11, 12, 32, 35, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_marked_as_duplicate.event, "marked_as_duplicate")
        self.assertEqual(self.event_marked_as_duplicate.id, 1783779725)
        self.assertEqual(self.event_marked_as_duplicate.issue.number, 857)
        self.assertEqual(
            self.event_marked_as_duplicate.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1783779725",
        )
        self.assertEqual(
            self.event_marked_as_duplicate.node_id,
            "MDIyOk1hcmtlZEFzRHVwbGljYXRlRXZlbnQxNzgzNzc5NzI1",
        )
        self.assertEqual(self.event_marked_as_duplicate.commit_url, None)
        self.assertEqual(self.event_marked_as_duplicate.label, None)
        self.assertEqual(self.event_marked_as_duplicate.assignee, None)
        self.assertEqual(self.event_marked_as_duplicate.assigner, None)
        self.assertEqual(self.event_marked_as_duplicate.author_association, None)
        self.assertEqual(self.event_marked_as_duplicate.review_requester, None)
        self.assertEqual(self.event_marked_as_duplicate.requested_reviewer, None)
        self.assertEqual(self.event_marked_as_duplicate.requested_team, None)
        self.assertEqual(self.event_marked_as_duplicate.milestone, None)
        self.assertEqual(self.event_marked_as_duplicate.performed_via_github_app, None)
        self.assertEqual(self.event_marked_as_duplicate.project_card, None)
        self.assertEqual(self.event_marked_as_duplicate.rename, None)
        self.assertEqual(self.event_marked_as_duplicate.dismissed_review, None)
        self.assertEqual(self.event_marked_as_duplicate.lock_reason, None)
        self.assertEqual(repr(self.event_marked_as_duplicate), "IssueEvent(id=1783779725)")

    def testEvent_unmarked_as_duplicate_Attributes(self):
        self.assertEqual(self.event_unmarked_as_duplicate.actor.login, "sfdye")
        self.assertEqual(self.event_unmarked_as_duplicate.commit_id, None)
        self.assertEqual(
            self.event_unmarked_as_duplicate.created_at,
            datetime(2018, 8, 15, 2, 57, 46, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_unmarked_as_duplicate.event, "unmarked_as_duplicate")
        self.assertEqual(self.event_unmarked_as_duplicate.id, 1789228962)
        self.assertEqual(self.event_unmarked_as_duplicate.issue.number, 857)
        self.assertEqual(
            self.event_unmarked_as_duplicate.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1789228962",
        )
        self.assertEqual(
            self.event_unmarked_as_duplicate.node_id,
            "MDI0OlVubWFya2VkQXNEdXBsaWNhdGVFdmVudDE3ODkyMjg5NjI=",
        )
        self.assertEqual(self.event_unmarked_as_duplicate.commit_url, None)
        self.assertEqual(self.event_unmarked_as_duplicate.label, None)
        self.assertEqual(self.event_unmarked_as_duplicate.assignee, None)
        self.assertEqual(self.event_unmarked_as_duplicate.assigner, None)
        self.assertEqual(self.event_unmarked_as_duplicate.author_association, None)
        self.assertEqual(self.event_unmarked_as_duplicate.review_requester, None)
        self.assertEqual(self.event_unmarked_as_duplicate.requested_reviewer, None)
        self.assertEqual(self.event_unmarked_as_duplicate.requested_team, None)
        self.assertEqual(self.event_unmarked_as_duplicate.milestone, None)
        self.assertEqual(self.event_unmarked_as_duplicate.performed_via_github_app, None)
        self.assertEqual(self.event_unmarked_as_duplicate.project_card, None)
        self.assertEqual(self.event_unmarked_as_duplicate.rename, None)
        self.assertEqual(self.event_unmarked_as_duplicate.dismissed_review, None)
        self.assertEqual(self.event_unmarked_as_duplicate.lock_reason, None)
        self.assertEqual(repr(self.event_unmarked_as_duplicate), "IssueEvent(id=1789228962)")

    def testEvent_added_to_project_Attributes(self):
        self.assertEqual(self.event_added_to_project.actor.login, "sfdye")
        self.assertEqual(self.event_added_to_project.commit_id, None)
        self.assertEqual(
            self.event_added_to_project.created_at,
            datetime(2018, 8, 16, 8, 13, 24, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_added_to_project.event, "added_to_project")
        self.assertEqual(self.event_added_to_project.id, 1791766828)
        self.assertEqual(self.event_added_to_project.issue.number, 857)
        self.assertEqual(
            self.event_added_to_project.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1791766828",
        )
        self.assertEqual(
            self.event_added_to_project.node_id,
            "MDE5OkFkZGVkVG9Qcm9qZWN0RXZlbnQxNzkxNzY2ODI4",
        )
        self.assertEqual(self.event_added_to_project.commit_url, None)
        self.assertEqual(self.event_added_to_project.label, None)
        self.assertEqual(self.event_added_to_project.assignee, None)
        self.assertEqual(self.event_added_to_project.assigner, None)
        self.assertEqual(self.event_added_to_project.author_association, None)
        self.assertEqual(self.event_added_to_project.review_requester, None)
        self.assertEqual(self.event_added_to_project.requested_reviewer, None)
        self.assertEqual(self.event_added_to_project.requested_team, None)
        self.assertEqual(self.event_added_to_project.milestone, None)
        self.assertEqual(self.event_added_to_project.performed_via_github_app, None)
        self.assertEqual(
            self.event_added_to_project.project_card,
            {
                "column_name": "To do",
                "id": 12179888,
                "project_id": 1714960,
                "project_url": "https://api.github.com/projects/1714960",
                "url": "https://api.github.com/projects/columns/cards/12179888",
            },
        )
        self.assertEqual(self.event_added_to_project.rename, None)
        self.assertEqual(self.event_added_to_project.dismissed_review, None)
        self.assertEqual(self.event_added_to_project.lock_reason, None)
        self.assertEqual(repr(self.event_added_to_project), "IssueEvent(id=1791766828)")

    def testEvent_moved_columns_in_project_Attributes(self):
        self.assertEqual(self.event_moved_columns_in_project.actor.login, "sfdye")
        self.assertEqual(self.event_moved_columns_in_project.commit_id, None)
        self.assertEqual(
            self.event_moved_columns_in_project.created_at,
            datetime(2018, 8, 16, 8, 13, 55, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_moved_columns_in_project.event, "moved_columns_in_project")
        self.assertEqual(self.event_moved_columns_in_project.id, 1791767766)
        self.assertEqual(self.event_moved_columns_in_project.issue.number, 857)
        self.assertEqual(
            self.event_moved_columns_in_project.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1791767766",
        )
        self.assertEqual(
            self.event_moved_columns_in_project.node_id,
            "MDI2Ok1vdmVkQ29sdW1uc0luUHJvamVjdEV2ZW50MTc5MTc2Nzc2Ng==",
        )
        self.assertEqual(self.event_moved_columns_in_project.commit_url, None)
        self.assertEqual(self.event_moved_columns_in_project.label, None)
        self.assertEqual(self.event_moved_columns_in_project.assignee, None)
        self.assertEqual(self.event_moved_columns_in_project.assigner, None)
        self.assertEqual(self.event_moved_columns_in_project.author_association, None)
        self.assertEqual(self.event_moved_columns_in_project.review_requester, None)
        self.assertEqual(self.event_moved_columns_in_project.requested_reviewer, None)
        self.assertEqual(self.event_moved_columns_in_project.requested_team, None)
        self.assertEqual(self.event_moved_columns_in_project.milestone, None)
        self.assertEqual(self.event_moved_columns_in_project.performed_via_github_app, None)
        self.assertEqual(
            self.event_moved_columns_in_project.project_card,
            {
                "column_name": "In progress",
                "id": 12179888,
                "previous_column_name": "To do",
                "project_id": 1714960,
                "project_url": "https://api.github.com/projects/1714960",
                "url": "https://api.github.com/projects/columns/cards/12179888",
            },
        )
        self.assertEqual(self.event_moved_columns_in_project.rename, None)
        self.assertEqual(self.event_moved_columns_in_project.dismissed_review, None)
        self.assertEqual(self.event_moved_columns_in_project.lock_reason, None)
        self.assertEqual(repr(self.event_moved_columns_in_project), "IssueEvent(id=1791767766)")

    def testEvent_removed_from_project_Attributes(self):
        self.assertEqual(self.event_removed_from_project.actor.login, "sfdye")
        self.assertEqual(self.event_removed_from_project.commit_id, None)
        self.assertEqual(
            self.event_removed_from_project.created_at,
            datetime(2018, 8, 16, 8, 14, 8, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_removed_from_project.event, "removed_from_project")
        self.assertEqual(self.event_removed_from_project.id, 1791768212)
        self.assertEqual(self.event_removed_from_project.issue.number, 857)
        self.assertEqual(
            self.event_removed_from_project.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1791768212",
        )
        self.assertEqual(
            self.event_removed_from_project.node_id,
            "MDIzOlJlbW92ZWRGcm9tUHJvamVjdEV2ZW50MTc5MTc2ODIxMg==",
        )
        self.assertEqual(self.event_removed_from_project.commit_url, None)
        self.assertEqual(self.event_removed_from_project.label, None)
        self.assertEqual(self.event_removed_from_project.assignee, None)
        self.assertEqual(self.event_removed_from_project.assigner, None)
        self.assertEqual(self.event_removed_from_project.author_association, None)
        self.assertEqual(self.event_removed_from_project.review_requester, None)
        self.assertEqual(self.event_removed_from_project.requested_reviewer, None)
        self.assertEqual(self.event_removed_from_project.requested_team, None)
        self.assertEqual(self.event_removed_from_project.milestone, None)
        self.assertEqual(self.event_removed_from_project.performed_via_github_app, None)
        self.assertEqual(
            self.event_removed_from_project.project_card,
            {
                "column_name": "In progress",
                "id": 12179888,
                "project_id": 1714960,
                "project_url": "https://api.github.com/projects/1714960",
                "url": "https://api.github.com/projects/columns/cards/12179888",
            },
        )
        self.assertEqual(self.event_removed_from_project.rename, None)
        self.assertEqual(self.event_removed_from_project.dismissed_review, None)
        self.assertEqual(self.event_removed_from_project.lock_reason, None)
        self.assertEqual(repr(self.event_removed_from_project), "IssueEvent(id=1791768212)")

    def testEvent_converted_note_to_issue_Attributes(self):
        self.assertEqual(self.event_converted_note_to_issue.actor.login, "sfdye")
        self.assertEqual(self.event_converted_note_to_issue.commit_id, None)
        self.assertEqual(
            self.event_converted_note_to_issue.created_at,
            datetime(2018, 8, 16, 8, 14, 34, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event_converted_note_to_issue.event, "converted_note_to_issue")
        self.assertEqual(self.event_converted_note_to_issue.id, 1791769149)
        self.assertEqual(self.event_converted_note_to_issue.issue.number, 866)
        self.assertEqual(
            self.event_converted_note_to_issue.url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/1791769149",
        )
        self.assertEqual(
            self.event_converted_note_to_issue.node_id,
            "MDI1OkNvbnZlcnRlZE5vdGVUb0lzc3VlRXZlbnQxNzkxNzY5MTQ5",
        )
        self.assertEqual(self.event_converted_note_to_issue.commit_url, None)
        self.assertEqual(self.event_converted_note_to_issue.label, None)
        self.assertEqual(self.event_converted_note_to_issue.assignee, None)
        self.assertEqual(self.event_converted_note_to_issue.assigner, None)
        self.assertEqual(self.event_converted_note_to_issue.author_association, None)
        self.assertEqual(self.event_converted_note_to_issue.review_requester, None)
        self.assertEqual(self.event_converted_note_to_issue.requested_reviewer, None)
        self.assertEqual(self.event_converted_note_to_issue.requested_team, None)
        self.assertEqual(self.event_converted_note_to_issue.milestone, None)
        self.assertEqual(self.event_converted_note_to_issue.performed_via_github_app, None)
        self.assertEqual(
            self.event_converted_note_to_issue.project_card,
            {
                "column_name": "To do",
                "id": 12179901,
                "project_id": 1714960,
                "project_url": "https://api.github.com/projects/1714960",
                "url": "https://api.github.com/projects/columns/cards/12179901",
            },
        )
        self.assertEqual(self.event_converted_note_to_issue.rename, None)
        self.assertEqual(self.event_converted_note_to_issue.dismissed_review, None)
        self.assertEqual(self.event_converted_note_to_issue.lock_reason, None)
        self.assertEqual(repr(self.event_converted_note_to_issue), "IssueEvent(id=1791769149)")



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Mahesh Raju <coder@mahesh.net>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2023 Thomas Burghout <thomas.burghout@nedap.com>                   #
# Copyright 2024 Benjamin K <53038537+treee111@users.noreply.github.com>       #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Nick McClorey <32378821+nickrmcclorey@users.noreply.github.com>#
# Copyright 2026 Denis Blanchette <dblanchette@coveo.com>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from github import GithubException

from . import Framework


class Workflow(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.workflow = self.g.get_repo("PyGithub/PyGithub").get_workflow("check.yml")

    def testAttributes(self):
        self.assertEqual(
            repr(self.workflow),
            'Workflow(url="https://api.github.com/repos/PyGithub/PyGithub/actions/workflows/1026390", name="check")',
        )
        self.assertEqual(self.workflow.badge_url, "https://github.com/PyGithub/PyGithub/workflows/check/badge.svg")
        self.assertEqual(self.workflow.created_at, datetime(2020, 4, 15, 0, 48, 32, tzinfo=timezone.utc))
        self.assertIsNone(self.workflow.deleted_at)
        self.assertEqual(
            self.workflow.html_url, "https://github.com/PyGithub/PyGithub/blob/master/.github/workflows/check.yml"
        )
        self.assertEqual(self.workflow.id, 1026390)
        self.assertEqual(self.workflow.name, "check")
        self.assertEqual(self.workflow.node_id, "MDg6V29ya2Zsb3cxMDI2Mzkw")
        self.assertEqual(self.workflow.path, ".github/workflows/check.yml")
        self.assertEqual(self.workflow.state, "active")
        timestamp = datetime(2020, 4, 15, 0, 48, 32, tzinfo=timezone.utc)
        self.assertEqual(self.workflow.created_at, timestamp)
        self.assertEqual(self.workflow.updated_at, timestamp)
        self.assertEqual(
            self.workflow.url,
            "https://api.github.com/repos/PyGithub/PyGithub/actions/workflows/1026390",
        )
        self.assertEqual(
            self.workflow.html_url,
            "https://github.com/PyGithub/PyGithub/blob/master/.github/workflows/check.yml",
        )
        self.assertEqual(
            self.workflow.badge_url,
            "https://github.com/PyGithub/PyGithub/workflows/check/badge.svg",
        )

    def testLazyAttributes(self):
        workflow = self.g.withLazy(True).get_repo("PyGithub/PyGithub").get_workflow("check.yml")
        self.assertEqual(
            str(workflow), 'Workflow(url="/repos/PyGithub/PyGithub/actions/workflows/check.yml", name=None)'
        )
        self.assertEqual(workflow.url, "/repos/PyGithub/PyGithub/actions/workflows/check.yml")

        workflow = self.g.withLazy(True).get_repo("PyGithub/PyGithub").get_workflow(42)
        self.assertEqual(str(workflow), 'Workflow(url="/repos/PyGithub/PyGithub/actions/workflows/42", name=None)')
        self.assertEqual(workflow.id, 42)
        self.assertEqual(workflow.url, "/repos/PyGithub/PyGithub/actions/workflows/42")

    def testGetRunsWithNoArguments(self):
        self.assertListKeyEqual(
            self.workflow.get_runs(),
            lambda r: r.id,
            [109950033, 109168419, 108934155, 108817672],
        )

    def testGetRunsWithObjects(self):
        sfdye = self.g.get_user("sfdye")
        master = self.g.get_repo("PyGithub/PyGithub").get_branch("master")
        self.assertListKeyEqual(
            self.workflow.get_runs(actor=sfdye, branch=master, event="push", status="completed"),
            lambda r: r.id,
            [100957683, 94845611, 93946842, 92714488],
        )

    def testGetRunsWithStrings(self):
        self.assertListKeyEqual(
            self.workflow.get_runs(actor="s-t-e-v-e-n-k", branch="master"),
            lambda r: r.id,
            [109950033, 108817672, 108794468, 107927403, 105213061, 105212023],
        )

    def testGetRunsWithHeadSha(self):
        self.assertListKeyEqual(
            self.workflow.get_runs(head_sha="3a6235b56eecc0e193c1e267b064c155c6ebc022"),
            lambda r: r.id,
            [3349872717],
        )

    def testGetRunsWithCreated(self):
        self.assertListKeyEqual(
            self.workflow.get_runs(created="2022-12-24"),
            lambda r: r.id,
            [3770390952],
        )

    def testCreateDispatchWithBranch(self):
        dispatch_inputs = {"logLevel": "Warning", "message": "Log Message"}
        workflow = self.g.get_repo("wrecker/PyGithub").get_workflow("manual_dispatch.yml")
        branch = self.g.get_repo("wrecker/PyGithub").get_branch("workflow_dispatch_branch")
        self.assertTrue(workflow.create_dispatch(branch, dispatch_inputs))

    def testCreateDispatchWithTag(self):
        dispatch_inputs = {"logLevel": "Warning", "message": "Log Message"}
        workflow = self.g.get_repo("wrecker/PyGithub").get_workflow("manual_dispatch.yml")
        tags = self.g.get_repo("wrecker/PyGithub").get_tags()
        tag = [t for t in tags if t.name == "workflow_dispatch_tag"].pop()
        self.assertTrue(workflow.create_dispatch(tag, dispatch_inputs))

    def testCreateDispatchWithString(self):
        dispatch_inputs = {"logLevel": "Warning", "message": "Log Message"}
        workflow = self.g.get_repo("wrecker/PyGithub").get_workflow("manual_dispatch.yml")
        ref_str = "main"
        self.assertTrue(workflow.create_dispatch(ref_str, dispatch_inputs))

    def testCreateDispatchForNonTriggerEnabled(self):
        workflow = self.g.get_repo("wrecker/PyGithub").get_workflow("check.yml")
        self.assertFalse(workflow.create_dispatch("main"))

    def testCreateDispatchException(self):
        workflow = self.g.get_repo("test-org/test-repo").get_workflow("workflow-with-params.yaml")
        with self.assertRaises(GithubException) as raisedexp:
            workflow.create_dispatch("main", throw=True)
        self.assertEqual(raisedexp.exception.status, 422)
        self.assertEqual(raisedexp.exception.data["message"], "Required input 'mandatory-parameter' not provided")

    def testDisable(self):
        workflow = self.g.get_repo("nickrmcclorey/PyGithub").get_workflow("ci.yml")
        self.assertTrue(workflow.disable())

    def testDisabledWhenAlreadyDisabled(self):
        workflow = self.g.get_repo("nickrmcclorey/PyGithub").get_workflow("ci.yml")
        self.assertFalse(workflow.disable())

    def testEnable(self):
        workflow = self.g.get_repo("nickrmcclorey/PyGithub").get_workflow("ci.yml")
        self.assertTrue(workflow.enable())

    def testEnableWhenAlreadyEnabled(self):
        workflow = self.g.get_repo("nickrmcclorey/PyGithub").get_workflow("ci.yml")
        self.assertTrue(workflow.enable())



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2018 Justin Kufro <jkufro@andrew.cmu.edu>                          #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class Traffic(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.user = self.g.get_user()
        self.repo = self.user.get_repo("PyGithub")

    def testGetReferrers(self):
        referrerResponse = self.repo.get_top_referrers()
        self.assertGreaterEqual(len(referrerResponse), 1)
        self.assertEqual(referrerResponse[0].uniques, 1)
        self.assertEqual(referrerResponse[0].referrer, "github.com")
        self.assertEqual(referrerResponse[0].count, 5)
        self.assertEqual(
            repr(referrerResponse[0]),
            'Referrer(uniques=1, referrer="github.com", count=5)',
        )

    def testGetPaths(self):
        pathsResponse = self.repo.get_top_paths()
        self.assertEqual(len(pathsResponse), 10)
        self.assertEqual(pathsResponse[0].uniques, 4)
        self.assertEqual(pathsResponse[0].count, 23)
        self.assertEqual(pathsResponse[0].path, "/jkufro/PyGithub")
        self.assertEqual(
            pathsResponse[0].title,
            "jkufro/PyGithub: Typed interactions with the GitHub API v3",
        )
        self.assertEqual(
            repr(pathsResponse[0]),
            'Path(uniques=4, title="jkufro/PyGithub: Typed interactions with the GitHub API v3", path="/jkufro/PyGithub", count=23)',
        )

    def testGetViews(self):
        views = self.repo.get_views_traffic()
        self.assertEqual(views.count, 93)
        self.assertEqual(views.uniques, 4)
        self.assertEqual(len(views.views), 5)
        view = views.views[0]
        self.assertEqual(view.uniques, 4)
        self.assertEqual(
            view.timestamp,
            datetime(2018, 11, 27, 0, 0, tzinfo=timezone.utc),
        )
        self.assertEqual(view.count, 56)
        self.assertEqual(
            repr(view),
            "Traffic(uniques=4, timestamp=2018-11-27 00:00:00+00:00, count=56)",
        )

    def testGetClones(self):
        clones = self.repo.get_clones_traffic()
        self.assertEqual(clones.count, 4)
        self.assertEqual(clones.uniques, 4)
        self.assertEqual(len(clones.clones), 1)
        clone = clones.clones[0]
        self.assertEqual(clone.uniques, 4)
        self.assertEqual(
            clone.timestamp,
            datetime(2018, 11, 27, 0, 0, tzinfo=timezone.utc),
        )
        self.assertEqual(clone.count, 4)
        self.assertEqual(
            repr(clone),
            "Traffic(uniques=4, timestamp=2018-11-27 00:00:00+00:00, count=4)",
        )



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2020 Dhruv Manilawala <dhruvmanila@gmail.com>                      #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class CheckSuite(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.check_suite_id = 1004503837
        self.test_check_suite_id = 1366665055
        self.test_repo = self.g.get_repo("dhruvmanila/pygithub-testing")
        self.test_check_suite = self.test_repo.get_check_suite(self.test_check_suite_id)
        self.repo = self.g.get_repo("wrecker/PySample")
        self.check_suite = self.repo.get_check_suite(self.check_suite_id)
        self.check_suite_ref = "fd09d934bcce792176d6b79d6d0387e938b62b7a"
        self.commit = self.repo.get_commit("fd09d934bcce792176d6b79d6d0387e938b62b7a")

    def testAttributes(self):
        cs = self.check_suite
        self.assertEqual(cs.after, "fd09d934bcce792176d6b79d6d0387e938b62b7a")
        self.assertEqual(cs.app.slug, "github-actions")
        self.assertEqual(cs.before, "9ee0caba8648aa0b8b5fc68ebc37c3c1162aa283")
        self.assertEqual(
            cs.check_runs_url,
            "https://api.github.com/repos/wrecker/PySample/check-suites/1004503837/check-runs",
        )
        self.assertEqual(cs.conclusion, "success")
        self.assertEqual(cs.created_at, datetime(2020, 8, 4, 5, 6, 54, tzinfo=timezone.utc))
        self.assertEqual(cs.head_branch, "wrecker-patch-1")
        self.assertEqual(cs.head_commit.sha, "fd09d934bcce792176d6b79d6d0387e938b62b7a")
        self.assertEqual(cs.head_sha, "fd09d934bcce792176d6b79d6d0387e938b62b7a")
        self.assertEqual(cs.id, self.check_suite_id)
        self.assertEqual(cs.latest_check_runs_count, 2)
        self.assertEqual(cs.id, self.check_suite_id)
        self.assertEqual(cs.node_id, "MDEwOkNoZWNrU3VpdGUxMDA0NTAzODM3")
        self.assertEqual(len(cs.pull_requests), 1)
        self.assertEqual(cs.pull_requests[0].id, 462527907)
        self.assertEqual(cs.repository.url, "https://api.github.com/repos/wrecker/PySample")
        self.assertEqual(cs.rerequestable, True)
        self.assertEqual(cs.runs_rerequestable, True)
        self.assertEqual(cs.status, "completed")
        self.assertEqual(cs.updated_at, datetime(2020, 8, 4, 5, 7, 40, tzinfo=timezone.utc))
        self.assertEqual(
            cs.url,
            "https://api.github.com/repos/wrecker/PySample/check-suites/1004503837",
        )

    def testLazyAttributes(self):
        suite = self.g.withLazy(True).get_repo("lazy/repo").get_check_suite(42)
        self.assertEqual(str(suite), 'CheckSuite(url="/repos/lazy/repo/check-suites/42", id=42)')
        self.assertEqual(suite.id, 42)
        self.assertEqual(suite.url, "/repos/lazy/repo/check-suites/42")

    def testGetCheckSuitesForRef(self):
        check_suites = self.commit.get_check_suites()
        self.assertEqual(check_suites.totalCount, 6)
        self.assertListEqual(
            [cs.id for cs in check_suites],
            [1004503392, 1004503393, 1004503395, 1004503397, 1004503837, 1004503857],
        )

    def testGetCheckSuitesForRefFilterByAppId(self):
        check_suites = self.commit.get_check_suites(app_id=29110)
        self.assertEqual(check_suites.totalCount, 1)
        self.assertListEqual([cs.id for cs in check_suites], [1004503392])

    def testGetCheckSuitesForRefFilterByCheckName(self):
        check_suites = self.commit.get_check_suites(check_name="Alex")
        self.assertEqual(check_suites.totalCount, 1)
        self.assertListEqual([cs.id for cs in check_suites], [1004503395])

    def testCheckSuiteRerequest(self):
        cs = self.repo.get_check_suite(1004503395)
        status = cs.rerequest()
        self.assertTrue(status)

    def testGetCheckRuns(self):
        check_runs = self.test_check_suite.get_check_runs()
        self.assertEqual(check_runs.totalCount, 8)
        self.assertListEqual(
            [cr.id for cr in check_runs],
            [
                1278952206,
                1279259090,
                1280450752,
                1280914700,
                1296027873,
                1296028076,
                1296029378,
                1296029552,
            ],
        )

    def testGetCheckRunsFilterByCheckName(self):
        check_runs = self.test_check_suite.get_check_runs(check_name="Testing")
        self.assertEqual(check_runs.totalCount, 1)
        self.assertEqual([cr.id for cr in check_runs], [1278952206])

    def testGetCheckRunsFilterByStatus(self):
        check_runs = self.test_check_suite.get_check_runs(status="completed")
        self.assertEqual(check_runs.totalCount, 8)
        self.assertListEqual(
            [cr.id for cr in check_runs],
            [
                1278952206,
                1279259090,
                1280450752,
                1280914700,
                1296027873,
                1296028076,
                1296029378,
                1296029552,
            ],
        )

    def testGetCheckRunsFilterByFilter(self):
        check_runs = self.test_check_suite.get_check_runs(filter="all")
        self.assertEqual(check_runs.totalCount, 8)
        self.assertListEqual(
            [cr.id for cr in check_runs],
            [
                1278952206,
                1279259090,
                1280450752,
                1280914700,
                1296027873,
                1296028076,
                1296029378,
                1296029552,
            ],
        )

    def testCreateCheckSuite(self):
        sha = "e5868bd5a9ccdd65c9c979250e11105f4c88faf4"
        check_suite = self.test_repo.create_check_suite(head_sha=sha)
        self.assertEqual(check_suite.head_sha, sha)
        self.assertEqual(check_suite.status, "queued")
        self.assertIsNone(check_suite.conclusion)

    def testUpdateCheckSuitesPreferences(self):
        data = [{"app_id": 85429, "setting": False}]
        repo_preferences = self.test_repo.update_check_suites_preferences(data)
        setting = None
        for app in repo_preferences.preferences["auto_trigger_checks"]:
            if app["app_id"] == data[0]["app_id"]:
                setting = app["setting"]
        self.assertFalse(setting)
        self.assertEqual(repo_preferences.repository.full_name, "dhruvmanila/pygithub-testing")
        data = [{"app_id": 85429, "setting": True}]
        repo_preferences = self.test_repo.update_check_suites_preferences(data)
        for app in repo_preferences.preferences["auto_trigger_checks"]:
            if app["app_id"] == data[0]["app_id"]:
                setting = app["setting"]
        self.assertTrue(setting)



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Rigas Papathanasopoulos <rigaspapas@gmail.com>                #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2023 chantra <chantra@users.noreply.github.com>                    #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from datetime import datetime, timezone
from unittest import mock

import github
from github.ApplicationOAuth import ApplicationOAuth as aoa

from . import Framework


class ApplicationOAuth(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.CLIENT_ID = "client_id_removed"
        self.CLIENT_SECRET = "client_secret_removed"
        self.app = self.g.get_oauth_application(self.CLIENT_ID, self.CLIENT_SECRET)
        self.ent_gh = github.Github(base_url="http://my.enterprise.com/path/to/github")
        self.ent_app = self.ent_gh.get_oauth_application(self.CLIENT_ID, self.CLIENT_SECRET)

    def testLoginURL(self):
        BASE_URL = "https://github.com/login/oauth/authorize"
        sample_uri = "https://myapp.com/some/path"
        sample_uri_encoded = "https%3A%2F%2Fmyapp.com%2Fsome%2Fpath"
        self.assertEqual(self.app.get_login_url(), f"{BASE_URL}?client_id={self.CLIENT_ID}")
        self.assertTrue(f"redirect_uri={sample_uri_encoded}" in self.app.get_login_url(redirect_uri=sample_uri))
        self.assertTrue(f"client_id={self.CLIENT_ID}" in self.app.get_login_url(redirect_uri=sample_uri))
        self.assertTrue("state=123abc" in self.app.get_login_url(state="123abc", login="user"))
        self.assertTrue("login=user" in self.app.get_login_url(state="123abc", login="user"))
        self.assertTrue(f"client_id={self.CLIENT_ID}" in self.app.get_login_url(state="123abc", login="user"))

    def testGetAccessToken(self):
        access_token = self.app.get_access_token("oauth_code_removed", state="state_removed")
        # Test string representation
        self.assertEqual(
            str(access_token),
            'AccessToken(type="bearer", token="acces...", scope="", '
            "refresh_token_expires_in=None, refresh_token=None, expires_in=None)",
        )
        self.assertEqual(access_token.token, "access_token_removed")
        self.assertEqual(access_token.type, "bearer")
        self.assertEqual(access_token.scope, "")
        self.assertIsNone(access_token.expires_in)
        self.assertIsNone(access_token.expires_at)
        self.assertIsNone(access_token.refresh_token)
        self.assertIsNone(access_token.refresh_expires_in)
        self.assertIsNone(access_token.refresh_expires_at)

    def testEnterpriseSupport(self):
        requester = self.ent_gh._Github__requester
        self.assertEqual(requester.scheme, "http")
        self.assertEqual(requester.hostname, "my.enterprise.com")
        self.assertEqual(requester.hostname_and_port, "my.enterprise.com")
        self.assertEqual(self.ent_app.get_oauth_url("auth"), "http://my.enterprise.com/login/oauth/auth")
        gh_w_port = github.Github(
            base_url="http://my.enterprise.com:443/path/to/github"
        )._Github__requester.hostname_and_port
        self.assertEqual(gh_w_port, "my.enterprise.com:443")

    def testEnterpriseLoginURL(self):
        BASE_URL = "http://my.enterprise.com/login/oauth/authorize"
        sample_uri = "https://myapp.com/some/path"
        sample_uri_encoded = "https%3A%2F%2Fmyapp.com%2Fsome%2Fpath"
        self.assertEqual(self.ent_app.get_login_url(), f"{BASE_URL}?client_id={self.CLIENT_ID}")
        self.assertTrue(f"redirect_uri={sample_uri_encoded}" in self.ent_app.get_login_url(redirect_uri=sample_uri))
        self.assertTrue(f"client_id={self.CLIENT_ID}" in self.ent_app.get_login_url(redirect_uri=sample_uri))
        self.assertTrue("state=123abc" in self.ent_app.get_login_url(state="123abc", login="user"))
        self.assertTrue("login=user" in self.ent_app.get_login_url(state="123abc", login="user"))
        self.assertTrue(f"client_id={self.CLIENT_ID}" in self.ent_app.get_login_url(state="123abc", login="user"))

    def testEnterpriseGetAccessToken(self):
        access_token = self.ent_app.get_access_token("oauth_code_removed", state="state_removed")
        # Test string representation
        self.assertEqual(
            str(access_token),
            'AccessToken(type="bearer", token="acces...", scope="", '
            "refresh_token_expires_in=None, refresh_token=None, expires_in=None)",
        )
        self.assertEqual(access_token.token, "access_token_removed")
        self.assertEqual(access_token.type, "bearer")
        self.assertEqual(access_token.scope, "")
        self.assertIsNone(access_token.expires_in)
        self.assertIsNone(access_token.expires_at)
        self.assertIsNone(access_token.refresh_token)
        self.assertIsNone(access_token.refresh_expires_in)
        self.assertIsNone(access_token.refresh_expires_at)

    def testGetAccessTokenWithExpiry(self):
        with mock.patch("github.AccessToken.datetime") as dt:
            dt.now = mock.Mock(return_value=datetime(2023, 6, 7, 12, 0, 0, 123, tzinfo=timezone.utc))
            access_token = self.app.get_access_token("oauth_code_removed", state="state_removed")
        # Test string representation
        self.assertEqual(
            str(access_token),
            'AccessToken(type="bearer", token="acces...", scope="", '
            'refresh_token_expires_in=15811200, refresh_token="refre...", expires_in=28800)',
        )
        self.assertEqual(access_token.token, "access_token_removed")
        self.assertEqual(access_token.type, "bearer")
        self.assertEqual(access_token.scope, "")
        self.assertEqual(access_token.expires_in, 28800)
        self.assertEqual(
            access_token.expires_at,
            datetime(2023, 6, 7, 20, 0, 0, 123, tzinfo=timezone.utc),
        )
        self.assertEqual(access_token.refresh_token, "refresh_token_removed")
        self.assertEqual(access_token.refresh_expires_in, 15811200)
        self.assertEqual(
            access_token.refresh_expires_at,
            datetime(2023, 12, 7, 12, 0, 0, 123, tzinfo=timezone.utc),
        )

    def testRefreshAccessToken(self):
        access_token = self.app.get_access_token("oauth_code_removed", state="state_removed")

        with mock.patch("github.AccessToken.datetime") as dt:
            dt.now = mock.Mock(return_value=datetime(2023, 6, 7, 12, 0, 0, 123, tzinfo=timezone.utc))
            refreshed = self.app.refresh_access_token(access_token.refresh_token)

        self.assertNotEqual(refreshed.token, access_token.token)
        self.assertNotEqual(refreshed.refresh_token, access_token.refresh_token)
        self.assertNotEqual(refreshed.created, access_token.created)
        # Test string representation
        self.assertEqual(
            str(refreshed),
            'AccessToken(type="bearer", token="anoth...", scope="", '
            'refresh_token_expires_in=15811200, refresh_token="anoth...", expires_in=28800)',
        )
        self.assertEqual(refreshed.token, "another_access_token_removed")
        self.assertEqual(refreshed.type, "bearer")
        self.assertEqual(refreshed.scope, "")
        self.assertEqual(
            refreshed.created,
            datetime(2023, 6, 7, 12, 0, 0, 123, tzinfo=timezone.utc),
        )
        self.assertEqual(refreshed.expires_in, 28800)
        self.assertEqual(
            refreshed.expires_at,
            datetime(2023, 6, 7, 20, 0, 0, 123, tzinfo=timezone.utc),
        )
        self.assertEqual(refreshed.refresh_token, "another_refresh_token_removed")
        self.assertEqual(refreshed.refresh_expires_in, 15811200)
        self.assertEqual(
            refreshed.refresh_expires_at,
            datetime(2023, 12, 7, 12, 0, 0, 123, tzinfo=timezone.utc),
        )

    def testGetAccessTokenBadCode(self):
        with self.assertRaises(github.BadCredentialsException) as exc:
            self.app.get_access_token("oauth_code_removed", state="state_removed")
        self.assertIsNone(exc.exception.message)
        self.assertEqual(exc.exception.status, 200)
        self.assertIn("error", exc.exception.data)
        self.assertEqual(exc.exception.data["error"], "bad_verification_code")

    def testGetAccessTokenUnknownError(self):
        with self.assertRaises(github.GithubException) as exc:
            self.app.get_access_token("oauth_code_removed", state="state_removed")
        self.assertIsNone(exc.exception.message)
        self.assertEqual(exc.exception.status, 200)
        self.assertIn("error", exc.exception.data)
        self.assertEqual(exc.exception.data["error"], "some_unknown_error")

    def testRefreshAccessTokenBadCode(self):
        with self.assertRaises(github.BadCredentialsException) as exc:
            self.app.refresh_access_token("oauth_code_removed")
        self.assertIsNone(exc.exception.message)
        self.assertEqual(exc.exception.status, 200)
        self.assertIn("error", exc.exception.data)
        self.assertEqual(exc.exception.data["error"], "bad_verification_code")

    def testRefreshAccessTokenUnknownError(self):
        with self.assertRaises(github.GithubException) as exc:
            self.app.refresh_access_token("oauth_code_removed")
        self.assertIsNone(exc.exception.message)
        self.assertEqual(exc.exception.status, 200)
        self.assertIn("error", exc.exception.data)
        self.assertEqual(exc.exception.data["error"], "some_unknown_error")

    def testCheckError(self):
        expected_header = {"header": True}
        expected_data = {"data": True}

        header, data = aoa._checkError(expected_header, None)
        self.assertEqual(header, expected_header)
        self.assertIsNone(data)

        header, data = aoa._checkError(expected_header, expected_data)
        self.assertEqual(header, expected_header)
        self.assertEqual(data, expected_data)

        with self.assertRaises(github.BadCredentialsException) as exc:
            aoa._checkError({}, {"error": "bad_verification_code"})
        self.assertIsNone(exc.exception.message)
        self.assertEqual(exc.exception.status, 200)
        self.assertIn("error", exc.exception.data)
        self.assertEqual(exc.exception.data["error"], "bad_verification_code")

        with self.assertRaises(github.GithubException) as exc:
            aoa._checkError({}, {"error": "other"})
        self.assertIsNone(exc.exception.message)
        self.assertEqual(exc.exception.status, 200)
        self.assertIn("error", exc.exception.data)
        self.assertEqual(exc.exception.data["error"], "other")



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 John Eskew <jeskew@edx.org>                                   #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Simon <spam@esemi.ru>                                         #
# Copyright 2018 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class CommitCombinedStatus(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.combined_status = (
            self.g.get_repo("edx/edx-platform", lazy=True)
            .get_commit("74e70119a23fa3ffb3db19d4590eccfebd72b659")
            .get_combined_status()
        )

    def testAttributes(self):
        self.assertEqual(self.combined_status.state, "success")
        self.assertEqual(
            self.combined_status.statuses[0].url,
            "https://api.github.com/repos/edx/edx-platform/statuses/74e70119a23fa3ffb3db19d4590eccfebd72b659",
        )
        self.assertEqual(self.combined_status.statuses[1].id, 390603044)
        self.assertEqual(self.combined_status.statuses[2].state, "success")
        self.assertEqual(self.combined_status.statuses[3].description, "Build finished.")
        self.assertEqual(
            self.combined_status.statuses[4].target_url,
            "https://build.testeng.edx.org/job/edx-platform-python-unittests-pr/10504/",
        )
        self.assertEqual(
            self.combined_status.statuses[4].created_at,
            datetime(2015, 12, 14, 13, 24, 18, tzinfo=timezone.utc),
        )
        self.assertEqual(
            self.combined_status.statuses[3].updated_at,
            datetime(2015, 12, 14, 13, 23, 35, tzinfo=timezone.utc),
        )
        self.assertEqual(self.combined_status.sha, "74e70119a23fa3ffb3db19d4590eccfebd72b659")
        self.assertEqual(self.combined_status.total_count, 6)
        self.assertEqual(self.combined_status.repository.id, 10391073)
        self.assertEqual(self.combined_status.repository.full_name, "edx/edx-platform")
        self.assertEqual(
            self.combined_status.commit_url,
            "https://api.github.com/repos/edx/edx-platform/commits/74e70119a23fa3ffb3db19d4590eccfebd72b659",
        )
        self.assertEqual(
            self.combined_status.url,
            "https://api.github.com/repos/edx/edx-platform/commits/74e70119a23fa3ffb3db19d4590eccfebd72b659/status",
        )
        self.assertEqual(
            repr(self.combined_status),
            'CommitCombinedStatus(state="success", sha="74e70119a23fa3ffb3db19d4590eccfebd72b659")',
        )



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

import github

from . import Framework


# Replay data for this test case is forged, because I don't have access to a real Github Enterprise install
class Enterprise(Framework.BasicTestCase):
    def testHttps(self):
        g = github.Github(auth=self.oauth_token, base_url="https://my.enterprise.com")
        self.assertListKeyEqual(
            g.get_user().get_repos(),
            lambda r: r.name,
            [
                "TestPyGithub",
                "django",
                "PyGithub",
                "developer.github.com",
                "acme-public-website",
                "C4Planner",
                "Hacking",
                "vincent-jacques.net",
                "Contests",
                "Candidates",
                "Tests",
                "DrawTurksHead",
                "DrawSyntax",
                "QuadProgMm",
                "Boost.HierarchicalEnum",
                "ViDE",
            ],
        )

    def testHttp(self):
        g = github.Github(auth=self.oauth_token, base_url="http://my.enterprise.com")
        self.assertListKeyEqual(
            g.get_user().get_repos(),
            lambda r: r.name,
            [
                "TestPyGithub",
                "django",
                "PyGithub",
                "developer.github.com",
                "acme-public-website",
                "C4Planner",
                "Hacking",
                "vincent-jacques.net",
                "Contests",
                "Candidates",
                "Tests",
                "DrawTurksHead",
                "DrawSyntax",
                "QuadProgMm",
                "Boost.HierarchicalEnum",
                "ViDE",
            ],
        )

    def testUnknownUrlScheme(self):
        with self.assertRaises(AssertionError) as raisedexp:
            github.Github(auth=self.oauth_token, base_url="foobar://my.enterprise.com")
        self.assertEqual(raisedexp.exception.args[0], "Unknown URL scheme")

    def testLongUrl(self):
        g = github.Github(auth=self.oauth_token, base_url="http://my.enterprise.com/path/to/github")
        repos = g.get_user().get_repos()
        self.assertListKeyEqual(
            repos,
            lambda r: r.name,
            [
                "TestPyGithub",
                "django",
                "PyGithub",
                "developer.github.com",
                "acme-public-website",
                "C4Planner",
                "Hacking",
                "vincent-jacques.net",
                "Contests",
                "Candidates",
                "Tests",
                "DrawTurksHead",
                "DrawSyntax",
                "QuadProgMm",
                "Boost.HierarchicalEnum",
                "ViDE",
            ],
        )
        self.assertEqual(repos[0].owner.name, "Vincent Jacques")

    def testSpecificPort(self):
        g = github.Github(auth=self.oauth_token, base_url="http://my.enterprise.com:8080")
        self.assertListKeyEqual(
            g.get_user().get_repos(),
            lambda r: r.name,
            [
                "TestPyGithub",
                "django",
                "PyGithub",
                "developer.github.com",
                "acme-public-website",
                "C4Planner",
                "Hacking",
                "vincent-jacques.net",
                "Contests",
                "Candidates",
                "Tests",
                "DrawTurksHead",
                "DrawSyntax",
                "QuadProgMm",
                "Boost.HierarchicalEnum",
                "ViDE",
            ],
        )
